; TERRAIN.ASM
; Handles checking bullets against tiles on the ground.

;==============================================================
; GO THROUGH ALL BULLETS AND CHECK AGAINST THE TERRAIN
;==============================================================

shoot_terrain:

	ld hl,blown_up_terrain
	ld (blown_up_ptr),hl

	ld a,7
	ld ix,bullets
-:	push af

	ld a,(ix+0)
	or (ix+1)
	jp z,not_bullet

	ld a,(ix+1)	
	cp 24
	jp c,not_bullet


check_same_bullet:

	; SO, IX POINTS TO A VALID BULLET


	ld b,(ix+0)
	ld a,(ix+1)
	add a,12
	ld c,a
	call get_name_table_loc

	; HL = pointer to spot on name table
	
	ld (name_table_destroy),hl

	; Divide HL by 2:
	srl h
	rr l

	; Now adjust so it's pointing to a spot in RAM:
	ld de,names_ram_copy
	add hl,de
	ld (name_ram_destroy),hl

	; Check which tile we hit:

	ld a,(hl)
	ld (tile_hit),a

	cp 3
	; It's the end-of-level EYE
	jr nz,not_shot_eye
	ld a,(shot_eyes)
	inc a
	ld (shot_eyes),a
	ld a,(tile_hit)
not_shot_eye:


	; OK, well, if it wasn't an eye (or even if it was)
	; let's look it up, see if we've got something right

	ld hl,destroyable_tiles
	ld bc,destroyable_tiles_end-destroyable_tiles
	cpir

	jp nz,not_bullet	; It wasn't a destroyable tile

	; So, we hit SOMETHING. We don't really care what it USED to be,
	; but more what we need to replace it with. Look that up based
	; on the index through the destroyable_tiles (returned above)


	; Do some quick resets...
	xor a
	ld (hit_wide_building),a

	; Queue the explosion sound
	exx
	push ix
	ld a,1
	call play_sound
	pop ix
	exx

	; Score!

	push hl


	ld de,destroyed_score-destroyable_tiles-1
	add hl,de
	ld a,(hl)

	push af
	call add_to_score
	pop af
	call add_to_score

	pop hl


	ld de,destroyed_tiles-destroyable_tiles-1
	add hl,de
	ld a,(hl)
	ld (tile_destroyed),a	; Save that away

	; Now we have that information...
	; Have we hit the RIGHT HAND SIDE of something?
	; Let's check - if we HAVE, then we'll be replacing the
	; current tile with either tile #27 or #31

	cp 27
	jr z,is_right_hand_tile
	cp 31
	jr nz,is_not_right_hand_tile

is_right_hand_tile:

	; Well, we HAVE hit the right hand side.
	; We'll need to use a different "blown up" tile, then:

	dec a
	ld (tile_destroyed),a

	; Magic. We'll also need to adjust our RAM pointer
	; and our VRAM pointers...

	ld hl,(name_ram_destroy)
	dec hl
	ld (name_ram_destroy),hl

	ld hl,(name_table_destroy)
	dec hl
	dec hl
	ld (name_table_destroy),hl

	ld a,(tile_hit)
	dec a
	ld (tile_hit),a

	ld a,1
	ld (hit_wide_building),a

	; Skip the wide building checks - we KNOW it is!
	jr not_wide_building

is_not_right_hand_tile:

	; Well, now we have that information...
	; Now, just because we have not hit the right hand 
	; side of things does NOT ALWAYS MEAN WE KNOW WE 
	; HAVE NOT HIT A WIDE TILE!

	ld a,(tile_hit)
	ld hl,wide_tiles
	ld bc,wide_tiles_end-wide_tiles
	cpir

	jr nz,not_wide_building

	ld a,1
	ld (hit_wide_building),a

not_wide_building:
	
	; So, we know know...
	; - WHERE IN RAM this building is.
	; - WHERE ON THE VDP this building is.
	; - WHETHER IT IS WIDE or not.
	; - WHICH TILE[S] TO REPLACE IT WITH.

	; Let's BLOW SOME SHIT UP
	
	call destroy_terrain

not_erase_old_building:

	ld de,64
	ld bc,32
	ld a,(tile_destroyed)
	cp 26
	jr z,hit_upper
	cp 30
	jr nz,not_hit_lower

hit_lower:
	scf
	ccf	
	ld hl,(name_table_destroy)
	sbc hl,de
	ld (name_table_destroy),hl
	scf
	ccf
	ld hl,(name_ram_destroy)
	sbc hl,bc
	ld (name_ram_destroy),hl
	ld a,26
	ld (tile_destroyed),a
	jr tall_blocks
hit_upper:
	ld hl,(name_table_destroy)
	add hl,de
	ld (name_table_destroy),hl
	ld hl,(name_ram_destroy)
	add hl,bc
	ld (name_ram_destroy),hl
	ld a,30
	ld (tile_destroyed),a
tall_blocks:

	; CHECK the range:

	ld hl,(name_ram_destroy)
	ld de,names_ram_copy
	call cp_hl_de

	jr nc,not_blown_off_top

	ld de,32*28
	add hl,de
	ld (name_ram_destroy),hl

	ld de,64*28
	ld hl,(name_table_destroy)
	add hl,de
	ld (name_table_destroy),hl	; <- was A... WTF?!
	jr safe_to_blow_up
not_blown_off_top:
	ld hl,(name_ram_destroy)
	ld de,names_ram_copy+32*28
	call cp_hl_de
	jr c,safe_to_blow_up

	ld de,-32*28
	add hl,de
	ld (name_ram_destroy),hl


	ld de,-64*28
	ld hl,(name_table_destroy)
	add hl,de
	ld (name_table_destroy),hl	; <- again, was A... ARGH!

safe_to_blow_up:
	call destroy_terrain

	
not_hit_lower:

	; Add an explosion:

	ld b,(ix+0)
	ld a,(ix+1)
	ld c,a
	call add_explosion_terrain

	; Erase the bullet itself:

	xor a
	ld (ix+0),a
	ld (ix+1),a


not_bullet:
	inc ix
	inc ix
	pop af
	dec a
	jp nz,-

	; Now we have finished, write 0 to the end of the buffer:

	ld hl,(blown_up_ptr)
	xor a
	ld (hl),a
	

	ret

destroy_terrain:
	ld hl,(name_table_destroy)
	ld de,vram_names
	add hl,de


.ifdef LIVE_UPDATE_TILES

	call vid_goto_address
	ld a,(tile_destroyed)
	out (vdp_data),a
	xor a
	out (vdp_data),a
.else

	push ix

	ld ix,(blown_up_ptr)
	ld a,(tile_destroyed)
	ld (ix+0),a
	ld (ix+2),h
	ld (ix+1),l
	inc ix
	inc ix
	inc ix
	ld (blown_up_ptr),ix

	pop ix

.endif

	ld hl,(name_ram_destroy)
	ld a,(tile_destroyed)
	ld (hl),a

	; Now we have destroyed a building, do we
	; need to kill the cell to the right?

	ld a,(hit_wide_building)
	or a
	ret z

.ifdef LIVE_UPDATE_TILES

	ld a,(tile_destroyed)
	inc a
	out (vdp_data),a
	xor a
	out (vdp_data),a

.else

	push ix
	

	ld a,(tile_destroyed)
	inc a


	ld ix,(blown_up_ptr)
	ld (ix+0),a

	ld hl,(name_table_destroy)
	ld de,vram_names+2
	add hl,de

	ld (ix+2),h
	ld (ix+1),l
	inc ix
	inc ix
	inc ix
	ld (blown_up_ptr),ix

	pop ix

.endif

	ld hl,(name_ram_destroy)
	inc hl
	ld (hl),a
	ret