; ZONES.ASM
; Displays zone name if applicable, bails out if not.

disp_zone_info:

	call screen_off

	call clear_sound

	ld hl,named_zones
	ld bc,named_zones_end-named_zones

	ld a,(level_num)

	cpir
	ret nz	; Not a named zone [should never happen!]


	ld de,zone_level_is_in-named_zones-1
	add hl,de

	ld a,(hl)
	ld l,a
	ld h,0


	; Each zone name = 18 long
	; 18 = 16+2
	add hl,hl
	ld d,h
	ld e,l
	add hl,hl
	add hl,hl
	add hl,hl
	add hl,de

	ld de,zone_names
	add hl,de

	; HL = pointer to zone name

	push hl

	; Load the BBC Micro font:
	call vid_clear_vram
	call load_beeb_font

	; Load the sprites:
	ld hl,grfx_sprites
	ld de,grfx_sprites_end-grfx_sprites
	ld a,2
	ld bc,256
	call vid_copy_tileset

	; Load the yellow/red sprite palette
	ld hl,sprite_palettes
	ld a,32
	ld b,8
	call vid_load_palette


	; Go to position to write zone name:

	xor a
	ld b,vdp_reg_scroll_y
	call vid_set_vdp_reg

	xor a
	call vid_goto_palette
	xor a
	out (vdp_data),a
	out (vdp_data),a
	out (vdp_data),a
	out (vdp_data),a

	ld a,34
	call vid_goto_palette
	xor a
	out (vdp_data),a
	out (vdp_data),a
	out (vdp_data),a
	out (vdp_data),a

	ld (game_clock),a
	
	ld hl,vram_names+64*10+12+2
	call vid_goto_address

	pop hl

	ld b,18

-:	ld a,(hl)
	sub $20
	out (vdp_data),a
	xor a
	out (vdp_data),a
	inc hl
	djnz -

	ld hl,vram_names+64*8+12
	call vid_goto_address

	ld hl,search_team_location
	ld b,20
-:	ld a,(hl)
	sub $20
	out (vdp_data),a
	xor a
	out (vdp_data),a
	inc hl
	djnz -

	ld hl,vram_names+64*4+12+10
	call vid_goto_address

	ld hl,fire_track
	ld b,10
-:	ld a,(hl)
	sub $20
	out (vdp_data),a
	xor a
	out (vdp_data),a
	inc hl
	djnz -


	ld hl,vram_names+64*16+12+18
	call vid_goto_address

	ld a,(num_lives)
	inc a
	call disp_a


	; Switch screen on
	call screen_on



spinning_ships_loop:
	call reset_sprites
	call vid_wait_vblank

	ld a,(game_clock)
	inc a
	cp 32
	jr z,+
	ld (game_clock),a

	call zone_set_palette

+:

	ld a,(num_lives)
	inc a
	ld b,a

	ld a,(helper_pos)
	inc a
	ld (helper_pos),a

	xor a
	ld (helper_ang),a

draw_ring_of_ships:

	push bc

	ld a,(num_lives)
	ld l,a
	ld h,0
	ld de,angle_offsets-1
	add hl,de
	ld a,(hl)
	ld c,a

	ld a,(helper_ang)
	add a,c
	ld (helper_ang),a
	ld c,a

	
	ld a,(helper_pos)
	add a,c
	and %01111111

	ld l,a
	ld h,0
	ld de,spinning_ship
	add hl,de
	ld a,(hl)
	add a,128
	ld b,a
	push bc

	
	ld a,(helper_pos)
	add a,c
	add a,32
	and %01111111
	ld l,a
	ld h,0
	ld de,spinning_ship
	add hl,de
	ld a,(hl)
	add a,80+44
	pop bc
	ld c,a
	
	ld d,0
	call draw_large_sprite

	pop bc
	djnz draw_ring_of_ships

	call finish_sprites
	call display_sprites
	in a,(start)
	and start_button
	jp nz,spinning_ships_loop

-:	call vid_wait_vblank

	ld a,(game_clock)
	dec a
	cp -1
	ret z
	ld (game_clock),a

	call zone_set_palette
	jr -

	ret

zone_set_palette:
	ld a,1
	call vid_goto_palette
	ld a,(game_clock)
	srl a
	ld b,a
	add a,a
	add a,a
	add a,a
	add a,a
	or b
	ld c,a
	out (vdp_data),a
	ld a,b
	out (vdp_data),a
	ld a,17
	call vid_goto_palette
	ld a,c
	out (vdp_data),a
	xor a
	out (vdp_data),a
	ld a,c
	and %00001111
	out (vdp_data),a
	xor a
	out (vdp_data),a
	ret

angle_offsets:
.db 64,42,32,26,21,18,16,14,13

spinning_ship:
.dbsin 0,128,360/128,24,0